==> base-0.0.0.tgz: package/lib/debug.js <==
/**
 * @fileoverview This is a console log utility which supports different log 
 * levels (debug, info, warn, error, log). It provides enhanced logging 
 * capabilities with features such as color-coding, labeling, source tracking 
 * and time-stamping. It is a part of the `basd` foundational library which 
 * @module Debug
 */

const _ = require('lodash')
const cwd = process.cwd()
const DEBUG = process.env.DEBUG || 'warn'

/**
 * A utility class for improved console logging with support for
 * debug, info, warn, error and log levels.
 */
class debug {
  /**
   * Logging levels with corresponding numbers.
   */
  static levels = {
    debug: 0,
    info: 1,
    warn: 2,
    error: 3,
    log: 4,
  }

  /**
   * Color mapping for different log levels.
   */
  static colorMap = {
    debug: 'white',
    info: 'cyan',
    warn: 'yellow',
    error: 'red',
    log: 'green',
  }

  /**
   * Corresponding color codes for terminal.
   */
  static colors = {
    black: 30,
    red: 31,
    green: 32,
    yellow: 33,
    blue: 34,
    magenta: 35,
    cyan: 36,
    white: 37,
  }

  /**
   * The current log level to be used.
   */
  static level = DEBUG
  static logger = null
  static showAll = false
  static showLabel = true
  static showSource = true
  static showTime = true
  static showErrors = false

  /**
   * Initializes a new debug instance.
   * @param {string} level - The logging level.
   * @param {...any} args - Additional arguments for the log.
   */
  constructor(level, ...args) {
    this.level = level
    this.levelInt = 4
    this.args = args
    this.logger = debug.logger
    this.showAll = debug.showAll
    this.showLabel = debug.showLabel
    this.showSource = debug.showSource
    this.showTime = debug.showTime
    this.showErrors = debug.showErrors
    this.logLevel = debug.level
    this.levels = debug.levels
    this.colorMap = debug.colorMap
    this.colors = debug.colors
    this.output = []
    this.result = _.last(args)
  }

  /**
   * Returns the color for the given log level.
   * @param {string} level - The log level.
   * @returns {string} - The color for the log level.
   */
  getColor(level) {
    return _.get(this.colorMap, level, this.defaultLevel)
  }

  /**
   * Returns the color code for the given log level.
   * @param {string} level - The log level.
   * @returns {number} - The color code for the log level.
   */
  getColorNum(level) {
    const color = this.getColor(level)
    return _.get(this.colors, color, this.defaultColor)
  }

  /**
   * Returns the caller's information (method, file path, line number, and column number).
   * @returns {string} - The caller's information.
   */
  getCaller() {
    try {
      throw new Error()
    } catch (e) {
      const stackLines = e.stack.split('\n')
      let lastFilePath = ''
      for (let ii = 1; ii < stackLines.length; ii++) { // Start from 1 to skip the current getCaller line
        const match = stackLines[ii].match(/^\s*at (.*?) \((.*):(\d+):(\d+)\)$/)
        if (match) {
          const [, methodName, filePath, lineNumber, columnNumber] = match
          // If the current file path is different from the last file path (debug.js), return the caller information
          if (filePath !== __filename && lastFilePath === __filename) {
            const relativeFilePath = filePath.replace(cwd + '/', '')
            return `${methodName} (${relativeFilePath}:${lineNumber}:${columnNumber})`
          }
          lastFilePath = filePath
        }
      }
    }
    // Default to an empty string if the caller couldn't be determined
    return ''
  }

  /**
   * Returns the current time in the format of the local time string.
   * @returns {string} - The current local time string.
   */
  getTime() {
    const date = new Date()
    return date.toLocaleTimeString()
  }

  /**
   * Parses the log level and arguments.
   */
  parse() {
    let level = this.level

    // When there are no arguments, use the level as a log message
    if (_.isEmpty(this.args)) {
      this.args = [level]
      level = _.isError(level) ? 'error' : 'log'
    }

    // If the level is not defined or invalid, treat it as a log message
    if (_.isUndefined(this.levels[level])) {
      this.args.unshift(level)
      level = 'log'
    }

    // Convert string levels to numeric levels
    let levelInt = _.isString(level) ? this.levels[level] : level

    // Find the name of the logging level
    level = _.findKey(this.levels, value => value === levelInt)
    
    // Default to 'log' if level name is not defined
    level = _.defaultTo(level, 'log')

    // Get the integer representation
    levelInt = this.levels[level]

    // If the level is 'error', show the error message instead of the error object
    if (level === 'error' && !this.showErrors) {
      let err = this.args[0]
      this.args[0] = _.get(err, 'message', err)
    }

    this.level = level
    this.levelInt = levelInt
    this.result = _.last(this.args)
  }

  /**
   * Formats the log message and adds it to the output.
   */
  format() {
    if (this.showLabel && this.level !== 'log') {
      const color = this.getColorNum(this.level)
      this.output.push(`\x1b[${color}m%s\x1b[0m`)
      this.output.push(`:: ${this.level} ::`)
    }
    this.args.forEach(arg => this.output.push(arg))
    if (this.showSource && (this.level !== 'log' || (this.logLevel === 'debug' && this.showAll))) {
      const initiator = this.getCaller()
      this.output.push(`\x1b[2m${initiator}${this.showTime ? ' ' + this.getTime() : ''}\x1b[0m`)
    }
  }

  /**
   * Executes the log command.
   */
  execute() {
    // If the current logging level is at least as severe as the message level, log the message
    if (this.levelInt >= _.get(this.levels, this.logLevel, this.levels.error))
      console.log.apply(console, this.output)
    // Use Winston logger if available
    if (this.logger) {
      const args = _.clone(this.args) // Avoid side effects if console and logger methods modify args
      this.logger[this.level].apply(this.logger, args)
    }
  }

  /**
   * Performs parsing, formatting and execution of log message.
   */
  print() {
    if (this.logLevel === 'mute' || this.level === 'mute')
      return
    this.parse()
    this.format()
    this.execute()
  }

  /**
   * Parses, formats and logs a message using the debug instance.
   * @param {...any} args - Arguments for the log.
   * @returns {void}
   */
  static print(...args) {
    return Reflect.construct(this, args).print()
  }

  /**
   * Parses, formats and logs a debug message.
   * @param {...any} args - Arguments for the log.
   * @returns {void}
   */
  static debug(...args) {
    return this.print('debug', ...args)
  }

  /**
   * Parses, formats and logs an info message.
   * @param {...any} args - Arguments for the log.
   * @returns {void}
   */
  static info(...args) {
    return this.print('info', ...args)
  }

  /**
   * Parses, formats and logs a warning message.
   * @param {...any} args - Arguments for the log.
   * @returns {debug} - The debug class.
   */
  static warn(...args) {
    this.print('warn', ...args)
    return this
  }

  /**
   * Parses, formats and logs an error message. 
   * If the first argument is a string, it will be converted to an Error object.
   * @param {...any} args - Arguments for the log.
   * @returns {debug} - The debug class.
   */
  static error(...args) {
    args[0] = !_.isError(args[0]) && _.isString(args[0]) ? new Error(args[0]) : args[0]
    this.print('error', ...args)
    return this
  }

  /**
   * Parses, formats and logs a generic log message.
   * @param {...any} args - Arguments for the log.
   * @returns {void}
   */
  static log(...args) {
    return this.print('log', ...args)
  }

  /**
   * Sets the value of a given key.
   * @param {string} key - The key to set.
   * @param {any} value - The value to set.
   * @returns {debug} - The debug class.
   */
  static set(key, value) {
    debug[`show${_.upperFirst(key)}`] = !!value
    return this
  }

  /**
   * Shows a given key.
   * @param {string} key - The key to show.
   * @param {boolean} [value=true] - The value to set.
   * @returns {debug} - The debug class.
   */
  static show(key, value = true) {
    return this.set(key, value)
  }

  /**
   * Hides a given key.
   * @param {string} key - The key to hide.
   * @param {boolean} [value=false] - The value to set.
   * @returns {debug} - The debug class.
   */
  static hide(key, value = false) {
    return this.set(key, value)
  }

  /**
   * Shows an error message.
   * @param {...any} args - Arguments for the error log.
   * @returns {debug} - The debug class.
   */
  static showError(...args) {
    const orig = this.showErrors
    this.show('errors')
    this.error(...args)
    this.set('errors', orig)
    return this
  }

  /**
   * Logs a message and returns the result.
   * @param {any} result - The result to return.
   * @param {...any} args - Arguments for the log.
   * @returns {any} - The result.
   */
  static return(result, ...args) {
    if (_.isEmpty(args))
      args.unshift(result)
    this.print(...args)
    return result
  }
}

module.exports = new Proxy(debug, {
  /**
   * Proxy handler for the `apply` trap.
   * @param {debug} target - The target debug instance.
   * @param {any} receiver - The receiver argument (usually the proxy itself).
   * @param {any[]} args - Arguments for the call.
   * @returns {void}
   */
  apply (target, receiver, args) {
    return target.print(...args)
  }
})

==> base-0.0.2.tgz: package/lib/debug.js <==
/**
 * @fileoverview This is a console log utility which supports different log 
 * levels (debug, info, warn, error, log). It provides enhanced logging 
 * capabilities with features such as color-coding, labeling, source tracking 
 * and time-stamping. It is a part of the `basd` foundational library which 
 * @module Debug
 */

const _ = require('lodash')
const cwd = process.cwd()
const DEBUG = process.env.DEBUG || 'warn'

/**
 * A utility class for improved console logging with support for
 * debug, info, warn, error and log levels.
 */
class debug {
  /**
   * Initializes a new debug instance.
   * @param {string} level - The logging level.
   * @param {...any} args - Additional arguments for the log.
   */
  constructor(level, ...args) {
    this.level = level
    this.levelInt = 4
    this.args = args
    this.logger = debug.logger
    this.showAll = debug.showAll
    this.showLabel = debug.showLabel
    this.showSource = debug.showSource
    this.showTime = debug.showTime
    this.showErrors = debug.showErrors
    this.logLevel = debug.level
    this.levels = debug.levels
    this.colorMap = debug.colorMap
    this.colors = debug.colors
    this.output = []
    this.result = _.last(args)
  }

  /**
   * Returns the color for the given log level.
   * @param {string} level - The log level.
   * @returns {string} - The color for the log level.
   */
  getColor(level) {
    return _.get(this.colorMap, level, this.defaultLevel)
  }

  /**
   * Returns the color code for the given log level.
   * @param {string} level - The log level.
   * @returns {number} - The color code for the log level.
   */
  getColorNum(level) {
    const color = this.getColor(level)
    return _.get(this.colors, color, this.defaultColor)
  }

  /**
   * Returns the caller's information (method, file path, line number, and column number).
   * @returns {string} - The caller's information.
   */
  getCaller() {
    try {
      throw new Error()
    } catch (e) {
      const stackLines = e.stack.split('\n')
      let lastFilePath = ''
      for (let ii = 1; ii < stackLines.length; ii++) { // Start from 1 to skip the current getCaller line
        const match = stackLines[ii].match(/^\s*at (.*?) \((.*):(\d+):(\d+)\)$/)
        if (match) {
          const [, methodName, filePath, lineNumber, columnNumber] = match
          // If the current file path is different from the last file path (debug.js), return the caller information
          if (filePath !== __filename && lastFilePath === __filename) {
            const relativeFilePath = filePath.replace(cwd + '/', '')
            return `${methodName} (${relativeFilePath}:${lineNumber}:${columnNumber})`
          }
          lastFilePath = filePath
        }
      }
    }
    // Default to an empty string if the caller couldn't be determined
    return ''
  }

  /**
   * Returns the current time in the format of the local time string.
   * @returns {string} - The current local time string.
   */
  getTime() {
    const date = new Date()
    return date.toLocaleTimeString()
  }

  /**
   * Parses the log level and arguments.
   */
  parse() {
    let level = this.level

    // When there are no arguments, use the level as a log message
    if (_.isEmpty(this.args)) {
      this.args = [level]
      level = _.isError(level) ? 'error' : 'log'
    }

    // If the level is not defined or invalid, treat it as a log message
    if (_.isUndefined(this.levels[level])) {
      this.args.unshift(level)
      level = 'log'
    }

    // Convert string levels to numeric levels
    let levelInt = _.isString(level) ? this.levels[level] : level

    // Find the name of the logging level
    level = _.findKey(this.levels, value => value === levelInt)
    
    // Default to 'log' if level name is not defined
    level = _.defaultTo(level, 'log')

    // Get the integer representation
    levelInt = this.levels[level]

    // If the level is 'error', show the error message instead of the error object
    if (level === 'error' && !this.showErrors) {
      let err = this.args[0]
      this.args[0] = _.get(err, 'message', err)
    }

    this.level = level
    this.levelInt = levelInt
    this.result = _.last(this.args)
  }

  /**
   * Formats the log message and adds it to the output.
   */
  format() {
    if (this.showLabel && this.level !== 'log') {
      const color = this.getColorNum(this.level)
      this.output.push(`\x1b[${color}m%s\x1b[0m`)
      this.output.push(`:: ${this.level} ::`)
    }
    this.args.forEach(arg => this.output.push(arg))
    if (this.showSource && (this.level !== 'log' || (this.logLevel === 'debug' && this.showAll))) {
      const initiator = this.getCaller()
      this.output.push(`\x1b[2m${initiator}${this.showTime ? ' ' + this.getTime() : ''}\x1b[0m`)
    }
  }

  /**
   * Executes the log command.
   */
  execute() {
    // If the current logging level is at least as severe as the message level, log the message
    if (this.levelInt >= _.get(this.levels, this.logLevel, this.levels.error))
      console.log.apply(console, this.output)
    // Use Winston logger if available
    if (this.logger) {
      const args = _.clone(this.args) // Avoid side effects if console and logger methods modify args
      this.logger[this.level].apply(this.logger, args)
    }
  }

  /**
   * Performs parsing, formatting and execution of log message.
   */
  print() {
    if (this.logLevel === 'mute' || this.level === 'mute')
      return
    this.parse()
    this.format()
    this.execute()
  }

  /**
   * Parses, formats and logs a message using the debug instance.
   * @param {...any} args - Arguments for the log.
   * @returns {void}
   */
  static print(...args) {
    return Reflect.construct(this, args).print()
  }

  /**
   * Parses, formats and logs a debug message.
   * @param {...any} args - Arguments for the log.
   * @returns {void}
   */
  static debug(...args) {
    return this.print('debug', ...args)
  }

  /**
   * Parses, formats and logs an info message.
   * @param {...any} args - Arguments for the log.
   * @returns {void}
   */
  static info(...args) {
    return this.print('info', ...args)
  }

  /**
   * Parses, formats and logs a warning message.
   * @param {...any} args - Arguments for the log.
   * @returns {debug} - The debug class.
   */
  static warn(...args) {
    this.print('warn', ...args)
    return this
  }

  /**
   * Parses, formats and logs an error message. 
   * If the first argument is a string, it will be converted to an Error object.
   * @param {...any} args - Arguments for the log.
   * @returns {debug} - The debug class.
   */
  static error(...args) {
    args[0] = !_.isError(args[0]) && _.isString(args[0]) ? new Error(args[0]) : args[0]
    this.print('error', ...args)
    return this
  }

  /**
   * Parses, formats and logs a generic log message.
   * @param {...any} args - Arguments for the log.
   * @returns {void}
   */
  static log(...args) {
    return this.print('log', ...args)
  }

  /**
   * Sets the value of a given key.
   * @param {string} key - The key to set.
   * @param {any} value - The value to set.
   * @returns {debug} - The debug class.
   */
  static set(key, value) {
    debug[`show${_.upperFirst(key)}`] = !!value
    return this
  }

  /**
   * Shows a given key.
   * @param {string} key - The key to show.
   * @param {boolean} [value=true] - The value to set.
   * @returns {debug} - The debug class.
   */
  static show(key, value = true) {
    return this.set(key, value)
  }

  /**
   * Hides a given key.
   * @param {string} key - The key to hide.
   * @param {boolean} [value=false] - The value to set.
   * @returns {debug} - The debug class.
   */
  static hide(key, value = false) {
    return this.set(key, value)
  }

  /**
   * Shows an error message.
   * @param {...any} args - Arguments for the error log.
   * @returns {debug} - The debug class.
   */
  static showError(...args) {
    const orig = this.showErrors
    this.show('errors')
    this.error(...args)
    this.set('errors', orig)
    return this
  }

  /**
   * Logs a message and returns the result.
   * @param {any} result - The result to return.
   * @param {...any} args - Arguments for the log.
   * @returns {any} - The result.
   */
  static return(result, ...args) {
    if (_.isEmpty(args))
      args.unshift(result)
    this.print(...args)
    return result
  }
}

/**
 * Logging levels with corresponding numbers.
 */
debug.levels = {
  debug: 0,
  info: 1,
  warn: 2,
  error: 3,
  log: 4,
}

/**
 * Color mapping for different log levels.
 */
debug.colorMap = {
  debug: 'white',
  info: 'cyan',
  warn: 'yellow',
  error: 'red',
  log: 'green',
}

/**
 * Corresponding color codes for terminal.
 */
debug.colors = {
  black: 30,
  red: 31,
  green: 32,
  yellow: 33,
  blue: 34,
  magenta: 35,
  cyan: 36,
  white: 37,
}

/**
 * The current log level to be used.
 */
debug.level = DEBUG
debug.logger = null
debug.showAll = false
debug.showLabel = true
debug.showSource = true
debug.showTime = true
debug.showErrors = false


module.exports = new Proxy(debug, {
  /**
   * Proxy handler for the `apply` trap.
   * @param {debug} target - The target debug instance.
   * @param {any} receiver - The receiver argument (usually the proxy itself).
   * @param {any[]} args - Arguments for the call.
   * @returns {void}
   */
  apply (target, receiver, args) {
    return target.print(...args)
  }
})
